<?php
/**
 * Theme footer module
 *
 * @author LeapIn Corp.
 * @since 1.0.0
 */
class LEAPIN_Footer
{

    public static $inst;
    public function __construct()
    {

        self::$inst = LEAPIN_Theme_Mod::get_instance();
        $this->registerActionHooks();
        return $this;
    }

    public function registerActionHooks()
    {

        add_action('widgets_init', array($this, 'registerWidgets'));
        add_action('customize_register', array($this, 'registerCustomizer'));
        add_filter('leapin_custom_css', array($this, 'set_bg_image_url'));
        return $this;
    }

    public function registerWidgets()
    {
        for($i = 1; $i < 5; $i++){
            register_sidebar(array(
                'name' => sprintf(esc_html__('Footer %s', 'pjax-blog'), $i),
                'id' => sprintf('footer-area-%s', $i),
                'description' => '',
                'before_widget' => '<aside id="%1$s" class="widget %2$s">',
                'after_widget' => '</aside>',
                'before_title' => '<h2 class="widget-title h6">',
                'after_title' => '</h2>',
            ));
        }
        return $this;
    }

    public function registerCustomizer($wp_customize)
    {

        $this->registerCustomizerSettings('footer', $wp_customize);

        return $this;
    }

    public function registerCustomizerSettings($file_handle = '', $wp_customize)
    {

        require_once get_template_directory() . '/inc/footer/customizer/' . sanitize_title($file_handle) . '.php';

        return $this;
    }

    public static function get_flex_basis(){
        $inst = LEAPIN_Theme_Mod::get_instance();
        $columns = $inst->get(LEAPIN_IDs::$leapin_footer_widget_columns, LEAPIN_Theme_Mod::$default_leapin_footer_widget_columns);

        $transport_columns = array(
            '1-columns' => '100%',
            '2-columns' => '50%',
            '3-columns' => '33%',
            '4-columns' => '25%'
        );

        return $transport_columns[$columns];
    }

    public static function hide_column($pos){
        $columns = self::$inst->get(LEAPIN_IDs::$leapin_footer_widget_columns, LEAPIN_Theme_Mod::$default_leapin_footer_widget_columns);

        $transport_columns = array(
            '1-columns' => 1,
            '2-columns' => 2,
            '3-columns' => 3,
            '4-columns' => 4
        );
        return $transport_columns[$columns] >= $pos ? true : false;
    }

    public static function set_bg_image_url($css)
    {
        $img = self::$inst->get(LEAPIN_IDs::$leapin_footer_bg_image);
        if(isset($img)){
            $css .= "#leapin_footer{background-image: url($img);}";
        }
        return $css;
    }
}

new LEAPIN_Footer();
