<?php

/**
 * Class LEAPIN_Header
 *
 * @author LeapIn Corp.
 * @since 1.0.0
 */
class LEAPIN_Header
{

    public function __construct()
    {

        require_once('navigation/class-leapin-navigation.php');
        require_once('navigation/class-menu-with-description.php');
        $this->registerNavigationMenus();
        $this->registerActionHooks();

        return $this;
    }

    /**
     * [registerActionHooks description]
     * @return [type] [description]
     */
    public function registerActionHooks()
    {

        add_action('customize_register', array($this, 'customizer'));
        add_action('after_setup_theme', array($this, 'registerNavigationMenus'));
        add_action('leapin_loc_on_main_header', array($this, 'getSearchIcon'));
        $this->defineLogoPosition();

        return $this;
    }

    /**
     * Register navigation menu
     * @return [type] [description]
     */
    public function registerNavigationMenus()
    {

        // wp menus
        $menus = array(
            'main-nav' => __('Main Header Bar', 'pjax-blog')   // main nav in header
        );

        $menus['mobile-nav'] = __('Bottom Right Menu Button (mainly for mobile)', 'pjax-blog'); // mobile menu

        if (leapin_is_pro()) {
            $menus['secondary-nav'] = __('Below Header Bar', 'pjax-blog'); // secondary menu in header
        }
        // registering wp3+ menus
        register_nav_menus($menus);

        return $this;
    }

    /**
     * [customiser description]
     * @return [type] [description]
     */
    public function customizer($wp_customize)
    {

        $this->registerCustomizerSettings('primary-menu', $wp_customize);
        $this->registerCustomizerSettings('mobile-menu', $wp_customize);
        if (leapin_is_pro()) {
            $this->registerCustomizerSettings('secondary-menu', $wp_customize);
        }

        return;
    }

    public function registerCustomizerSettings($file_handle = '', $wp_customize)
    {

        require_once get_template_directory() . '/inc/header/customizer/' . sanitize_title($file_handle) . '.php';

        return $this;
    }

    public function defineLogoPosition()
    {
        // if main nav is not registered and search icon is hidden.
        if (!has_nav_menu('main-nav') && !leapin_is_mod_visible(LEAPIN_IDs::$leapin_hide_primary_menu_search_icon) && !leapin_is_mod_visible(LEAPIN_IDs::$leapin_feature_hb_hide)) {
            add_filter('leapin_custom_css', function ($css) {
                // centering logo horizontally
                $css .= 'header .logo,header .logo-typo, header .logo #site-description, header .logo img, header .logo p{text-align: center;} .main-header-inner{display:none;}';
                return $css;
            });
        }
    }

    public static function getMenuType()
    {
        return LEAPIN_Theme_Mod::get_instance()->get(LEAPIN_IDs::$leapin_secondary_menu_type, LEAPIN_Theme_Mod::$default_leapin_secondary_menu_type);
    }

    // get logo
    public static function leapin_logo($position = '')
    {
        $inst = LEAPIN_Theme_Mod::get_instance();
        $desktop_logo = false;
        if (has_custom_logo()) {
            $desktop_logo = get_custom_logo();
        }
        $mobile_logo = $inst->get(LEAPIN_IDs::$leapin_logo_mobile);
        $is_visible_desc = leapin_is_mod_visible(LEAPIN_IDs::$leapin_hide_site_description);

        $hide_logo_typo = '';
        $hide_mobile_logo_typo = '';

        ob_start();
        printf('<div id="logo-%s" class="logo%s">', $position, $is_visible_desc ? ' with-desc' : '');
        if ($desktop_logo) {
            // logo image is set
            $hide_logo_typo = 'hide_logo_typo';
            // echo desktop logo
            printf('<a href="%s" rel="nofollow"><img class="desktop-logo" src="%s" /></a>', leapin_get_home_url(), $desktop_logo);
        }
        if ($mobile_logo) {
            $hide_mobile_logo_typo = ' hide_mobile_logo_typo';
            $img = sprintf('<img class="mobile-logo" src="%s" />', $mobile_logo);
//            $img = LEAPIN_AMP_Tags::is_amp() ? sprintf('<amp-img class="mobile-logo" layout="fixed-height" src="%s" height="80"></amp-img>', $mobile_logo) : sprintf('<img class="mobile-logo" src="%s" />', $mobile_logo);
            printf('<a href="%s" rel="nofollow">%s</a>', leapin_get_home_url(), $img);
        }
        ?>
        <?php if ((is_home() || is_front_page()) && $position == 'header'): ?>
        <h1 id="logo-typo-<?php echo $position ?>"
            class="logo-typo <?php echo $hide_logo_typo . $hide_mobile_logo_typo ?>" itemscope
            itemtype="http://schema.org/Organization">
            <a href="<?php echo leapin_get_home_url(); ?>" rel="nofollow"><?php bloginfo('name'); ?></a>
        </h1>
    <?php else: ?>
        <p id="logo-typo-<?php echo $position ?>"
           class="logo-typo h1 <?php echo $hide_logo_typo . $hide_mobile_logo_typo ?>" itemscope
           itemtype="http://schema.org/Organization">
            <a href="<?php echo leapin_get_home_url(); ?>" rel="nofollow"><?php bloginfo('name'); ?></a>
        </p>
    <?php endif; ?>
        <?php if ($is_visible_desc): ?>
        <p id="site-description"><?php bloginfo('description'); ?></p>
    <?php endif; ?>
        <?php echo '</div>';
        return ob_get_clean();
    }

    public static function get_main_nav($position = 'header')
    {
        $fix = $position == 'fixed' ? ' fixed' : ' rel';
        ob_start();
        ?>
        <div class="main-header cf<?php echo $fix; ?>">
            <?php echo LEAPIN_Header::leapin_logo($position); ?>
            <?php $theme_location = 'main-nav'; ?>
            <div class="main-header-inner">
                <?php
                if (has_nav_menu($theme_location) || leapin_is_mod_visible(LEAPIN_IDs::$leapin_hide_primary_menu_search_icon)) :?>
                    <nav class="primary-nav" aria-label="Navigation" itemscope
                         itemtype="http://schema.org/SiteNavigationElement">
                        <?php
                        if (has_nav_menu($theme_location)):
                            wp_nav_menu(array(
                                'container' => false,                           // remove nav container
                                'container_class' => 'menu cf',                 // class of container (should you choose to use it)
                                'menu_class' => 'nav main-nav simple cf',              // adding custom nav class
                                'theme_location' => $theme_location,                 // where it's located in the theme
                                'link_before' => '<span>',                            // before each link
                                'link_after' => '</span>',                             // after each link
                                'depth' => 3,                                   // limit the depth of the nav
                                'walker' => new Menu_With_Description
                            ));
                        endif;
                        ?>
                    </nav>
                <?php endif; ?>
                <?php do_action('leapin_loc_on_main_header'); ?>
            </div>
        </div>
        <?php
        return ob_get_clean();
    }

    public function getSearchIcon(){
        if (leapin_is_mod_visible(LEAPIN_IDs::$leapin_hide_primary_menu_search_icon)) {
            get_search_form();
        }
    }
}

// Start
$tpHeader = new LEAPIN_Header();
