<?php

/**
 * Class LEAPIN_Query
 *
 * @author LeapIn Corp.
 * @since 1.0.0
 */
class LEAPIN_Query
{
    private $instance;
    private $post_type;
    private $post_ids;
    private $post_id;
    private $posts_per_page;
    private $theme_color;
    private $grayscale;
    private $thumbnail_size;
    private $related_post;
    private $paged;
    private $args;
    private $the_query;
    private $ajax;
    private $exclude_post_ids;
    private $term_ids_or_slugs;
    private $custom_fields;
    private $more;
    private $taxonomy;
    private $order_by;
    private $call_type;
    private $year;
    private $month;
    private $day;
    private $s;
    private $author;
    private $call_position;
    private $widget_args = [];
    public static $LEAPIN_POSTS = 'leapin_posts';
    public static $LEAPIN_NAV_TAXONOMY = 'leapin_nav_taxonomy';
    public static $LEAPIN_NAV_SINGLE = 'leapin_nav_single';
    public static $LEAPIN_NAV_THUMBNAIL = 'leapin_nav_thumbnail';
    public static $CALL_TYPE;

    public function __construct($instance = false, $args = [], $the_query = false)
    {
        $this->set_post_params($instance);
        $this->set_related_query_args($instance);
        $this->set_widget_args($args);
        $this->query_args_factory($the_query);
    }

    public function ajax_or_print()
    {
        // ajax call
        if ($this->ajax) {
            if (leapin_is_barba_call()) {
                $this->instance['ajax_on_load_call'] = 0;
                $this->the_leapin_posts();
            } else {
                $this->instance['ajax_on_load_call'] = 1;
                $this->ajax_call_tag();
            }
        } else {
            $this->instance['ajax_on_load_call'] = 0;
            $this->the_leapin_posts();
        }
    }

    public function ajax_call_tag()
    {
        echo $this->get_leapin_posts_container_before_tag();
        ?>
        <?php
        echo '</div></div>';
        echo $this->widget_args['after_widget'];
    }

    public function the_leapin_taxonomy()
    {
        $rtn = ['content' => ''];
        $the_query = new WP_Query($this->args);
        $i = 0;

        if ($the_query->have_posts()) {
            while ($the_query->have_posts()) : $the_query->the_post();
                $i++;
                $rtn['content'] .= LEAPIN_Query_Helper::get_nav_taxonomy_posts($this->instance, $the_query);
            endwhile;
        }
        wp_reset_postdata();

        $rtn['post_count'] = $i;
        $rtn['content'] = apply_filters('leapin_post_widget_return_content_json', $rtn['content']);
        return json_encode($rtn);
    }

    public function the_leapin_single()
    {
        $rtn = ['content' => ''];
        $the_query = new WP_Query($this->args);
        $i = 0;

        if ($the_query->have_posts()) {
            while ($the_query->have_posts()) : $the_query->the_post();
                $i++;
                $rtn['content'] .= LEAPIN_Query_Helper::get_nav_single($this->instance, $the_query);
            endwhile;
        }
        wp_reset_postdata();

        $rtn['post_count'] = $i;
        $rtn['content'] = apply_filters('leapin_post_widget_return_content_json', $rtn['content']);
        return json_encode($rtn);
    }

    public function the_leapin_posts($ajax_call = false)
    {
        $rtn = $this->get_leapin_posts($ajax_call);
        if ($ajax_call) {
            return json_encode($rtn);
        } else {
            echo $rtn['content'];
        }
    }

    private function get_leapin_posts_container_before_tag()
    {

        $val = $this->widget_args['before_widget'];
        $val .= sprintf('<div id="inner-%s"', $this->widget_args['widget_id']);
        foreach ($this->instance as $key => $value) {
            if (($value && !is_array($value)) || $value === 0) {
                if ($key == 'paged') {
                    if ($this->ajax && !leapin_is_barba_call()) {
                        $val .= sprintf(' data-%s="%s"', $key, 0);
                    } else {
                        $val .= sprintf(' data-%s="%s"', $key, 1);
                    }
                } else {
                    $val .= sprintf(' data-%s="%s"', $key, $value);
                }

            }
        }
        $val .= ' data-ajax="leapin_posts"';
        $val .= '>';
        $title = (!empty($this->instance['title'])) ? $this->instance['title'] : false;
        if ($title) {
            $val .= $this->widget_args['before_title'] . $title . $this->widget_args['after_title'];
        }

        $widget_id = $this->widget_args['widget_id'];
        $grayscale = isset($this->instance['grayscale']) ? $this->instance['grayscale'] : false;
        $theme_color = isset($this->instance['theme_color']) ? $this->instance['theme_color'] : false;
        $layout_type = isset($this->instance['layout_type']) ? $this->instance['layout_type'] : false;
        $this->set_custom_css($widget_id, $theme_color, $grayscale, $layout_type);
        return $val . '<div class="post_cards_container">';
    }

    private function set_custom_css($widget_id, $theme_color, $grayscale, $layout_type)
    {
        if (isset($widget_id)) {
            // set grayscale of thumbnail
            if (isset($grayscale) && $grayscale !== '0') {
                add_filter('leapin_load_css_on_each_pjax', function ($css) use ($widget_id, $grayscale) {
                    $css .= "#inner-$widget_id .card__thumbnail::after{filter: grayscale($grayscale%);}";
                    return $css;
                });
            }
            if (isset($theme_color)) {
                add_filter('leapin_load_css_on_each_pjax', function ($css) use ($widget_id, $theme_color, $layout_type) {
                    $lighten_theme_color = leapin_adjust_color_brightness($theme_color, 70);
                    $rgba3_theme_color = leapin_hex2rgba($theme_color, 0.3);
                    $rgba8_theme_color = leapin_hex2rgba($theme_color, 0.8);

                    if ($layout_type == 'swiper_ver1' || $layout_type == 'swiper_ver3' || $layout_type == 'swiper_ver4') {
                        $css .= "#inner-$widget_id .card__more {background-color: $theme_color;}";
                        $css .= "#inner-$widget_id .card__header::after {background-color: $lighten_theme_color;}";
                    }
                    if ($layout_type == 'swiper_ver1' || $layout_type == 'swiper_ver3') {
//                        $css .= "#inner-$widget_id .card__taxonomy a {color: $lighten_theme_color;}";
                    }
                    if ($layout_type == 'swiper_ver4') {
                        $css .= "#inner-$widget_id .card__taxonomy a {color: $theme_color;}";
                        $css .= "#inner-$widget_id .post_cards_container::before {background-color: $lighten_theme_color;}";
                    }
                    if ($layout_type == 'tile_ver1'
                        || $layout_type == 'tile_ver2'
                        || $layout_type == 'tile_ver3'
                        || $layout_type == 'tile_ver4'
                        || $layout_type == 'tile_ver5'
                        || $layout_type == 'tile_ver6'
                        || $layout_type == 'swiper_ver2'
                        || $layout_type == 'sidebar_ver1'
                    ) {
                        $css .= "#inner-$widget_id .card__taxonomy a {color: $theme_color;}";
                    }
                    if($layout_type == 'tile_ver3' || $layout_type == 'tile_ver6'){
                        $css .= "#inner-$widget_id .ajax-more-posts {background-color: $rgba3_theme_color;}";
                        $css .= "#inner-$widget_id .ajax-more-posts:hover {background-color: $rgba8_theme_color;}";
                    }
                    return $css;
                });
            }
        }
    }

    public function get_leapin_posts($ajax_call = false)
    {
        $rtn = ['content' => ''];
        $paged = $this->instance['paged'];
        $layout_type = isset($this->instance['layout_type']) ? $this->instance['layout_type'] : false;
        $the_query = is_object($this->the_query) ? $this->the_query : new WP_Query($this->args);

        if ($the_query->have_posts()) {
            if (!$ajax_call && !$paged) {
                $rtn['content'] .= $this->get_leapin_posts_container_before_tag();
            }

            if ($layout_type == 'swiper_ver4') {
                while ($the_query->have_posts()) : $the_query->the_post();

                    $rtn['content'] .= LEAPIN_Query_Helper::get_post_background($this->instance, $the_query);

                endwhile;
            }
            while ($the_query->have_posts()) : $the_query->the_post();

                $rtn['content'] .= LEAPIN_Query_Helper::get_post_cards($this->instance, $the_query);

            endwhile;

            $rtn['is_more_posts'] = self::is_more_posts($the_query, $this->instance);
            $rtn['post_count'] = $the_query->post_count;
            if (!$ajax_call && !$paged) {
                $rtn['content'] .= '</div>';
                $rtn['content'] .= '</div>';
                $rtn['content'] .= $this->widget_args['after_widget'];
            }
        }
        wp_reset_postdata();

        $rtn['content'] = apply_filters('leapin_post_widget_return_content_json', $rtn['content']);
        return $rtn;
    }

    public function get_leapin_thumbnail_url()
    {
        $rtn = [];
        $size = $this->thumbnail_size;
        foreach ($this->post_ids as $id) {
            $rtn[$id] = leapin_get_eyecatch_image($size, $id)[0];
        }
        return json_encode($rtn);
    }

    public static function is_first_loop($the_query)
    {
        return $the_query->current_post == 0;
    }

    public static function is_last_loop($the_query)
    {
        return ($the_query->current_post + 1) == $the_query->post_count;
    }

    public static function is_more_posts($the_query, $instance)
    {
        $posts_per_page = isset($instance['posts_per_page']) ? $instance['posts_per_page'] : false;
        return $the_query->post_count >= $posts_per_page;
    }

    private function set_post_params($instance)
    {
        $instance = is_array($instance) ? $instance : $_POST;
        $this->instance = $instance;

        $this->call_type = isset($instance['call_type']) ? $instance['call_type'] : false;
        self::$CALL_TYPE = $this->call_type;

        $this->posts_per_page = isset($instance['posts_per_page']) ? $instance['posts_per_page'] : 5;
        $this->theme_color = isset($instance['theme_color']) ? $instance['theme_color'] : false;
        $this->grayscale = isset($instance['grayscale']) && $instance['grayscale'] !== 0 ? $instance['grayscale'] : 0;
        $this->post_id = isset($instance['post_id']) ? $instance['post_id'] : false;
        $this->ajax = isset($instance['ajax']) ? $instance['ajax'] : false;
        $this->related_post = isset($instance['related_post']) ? $instance['related_post'] : false;
        $this->call_position = isset($instance['call_position']) ? $instance['call_position'] : false;
        $this->paged = $this->instance['paged'] = isset($instance['paged']) ? $instance['paged'] : 0;
        $this->year = isset($instance['year']) ? $instance['year'] : false;
        $this->month = isset($instance['month']) ? $instance['month'] : false;
        $this->day = isset($instance['day']) ? $instance['day'] : false;
        $this->author = isset($instance['author']) ? $instance['author'] : false;
        $this->s = isset($instance['s']) ? $instance['s'] : '';
        $this->custom_fields = isset($instance['custom_fields']) ? $instance['custom_fields'] : '';
        $this->thumbnail_size = isset($instance['thumbnail_size']) ? $instance['thumbnail_size'] : "leapin-medium";
        $this->post_ids = !empty($instance['post_ids']) ? explode(",", $instance['post_ids']) : [];
        foreach ($this->post_ids as &$item) $item = trim($item);
        $this->exclude_post_ids = !empty($instance['exclude_post_ids']) ? explode(",", $instance['exclude_post_ids']) : [];
        $this->taxonomy = isset($instance['taxonomy']) ? $instance['taxonomy'] : '';
        foreach ($this->exclude_post_ids as &$item) $item = trim($item);
        $this->term_ids_or_slugs = !empty($instance['term_ids_or_slugs']) ? explode(",", $instance['term_ids_or_slugs']) : [];
        foreach ($this->term_ids_or_slugs as &$item) $item = trim($item);
        $this->more = isset($instance['more']) ? (bool)$instance['more'] : false;
        $this->post_type = isset($instance['post_type']) ? $instance['post_type'] : false;
        $this->order_by = isset($instance['order_by']) ? $instance['order_by'] : LEAPIN_Query_Helper::$default_order_by;
    }

    private function set_widget_args($args)
    {

        $this->widget_args = array_merge(['before_widget' => LEAPIN_Query_Helper::$BEFORE_WIDGET,
                'after_widget' => LEAPIN_Query_Helper::$AFTER_WIDGET,
                'before_title' => LEAPIN_Query_Helper::$BEFORE_WIDGET_TITLE,
                'after_title' => LEAPIN_Query_Helper::$AFTER_WIDGET_TITLE,
                'widget_id' => LEAPIN_Query_Helper::$WIDGET_DUMMY_ID,
            ]
            , $args);
        $this->instance['widget_args'] = $this->widget_args;
    }

    private function query_args_factory($the_query)
    {
        if (is_object($the_query)) {
            $this->the_query = $the_query;
            return;
        }
        $this->args = [
            'p' => $this->post_id,
            'post_type' => $this->post_type,
            'post__in' => $this->post_ids,
            'post__not_in' => $this->exclude_post_ids,
            'post_status' => 'publish',
            'no_found_rows' => true,
            'cache_results' => true,
            'ignore_sticky_posts' => true,
            'posts_per_page' => $this->posts_per_page,
            'offset' => (int)$this->posts_per_page * (int)$this->paged,
            'order' => 'DESC',
            'orderby' => $this->order_by
        ];
        if ($this->year)
            $this->args['year'] = $this->year;
        if ($this->month)
            $this->args['monthnum'] = $this->month;
        if ($this->day)
            $this->args['day'] = $this->day;
        if ($this->author)
            $this->args['author'] = $this->author;
        if ($this->s)
            $this->args['s'] = $this->s;


        $this->args['tax_query'] = $this->get_tax_query();
        $this->args['meta_query'] = $this->get_meta_query();
        return apply_filters('leapin_post_widge_query_args', $this->args, $the_query, $this);
    }

    private function get_meta_query()
    {
        $meta_query = [];
        $meta_arry = [];
        if (empty($this->custom_fields))
            return $meta_query;

        $cf = explode('=', $this->custom_fields);
        if (count($cf) > 0) {
            $meta_arry['key'] = trim($cf[0]);
            $values = explode(",", $cf[1]);
            $meta_arry['value'] = [];
            foreach ($values as $val) {
                $meta_arry['value'][] = trim($val);
            }
            $meta_arry['compare'] = 'IN';
            $meta_query[] = $meta_arry;
        }

        return $meta_query;
    }

    private function get_tax_query()
    {
        $tax_query = [];
        $is_term_id = false;
        if ($this->taxonomy == "" || count($this->term_ids_or_slugs) == 0)
            return $tax_query;

        foreach ($this->term_ids_or_slugs as $terms) {
            if (is_numeric($terms)) {
                $is_term_id = true;
            }
        }

        $tax_query[] = [
            'taxonomy' => $this->taxonomy,
            'field' => $is_term_id ? 'term_id' : 'slug',
            'terms' => $this->term_ids_or_slugs
        ];

        return $tax_query;
    }

    private function set_related_query_args($instance)
    {
        if ($this->related_post && is_single()) {
            $p = get_post();
            if (is_object($p)) {
                $terms = [];
                $wp_terms = [];
                $tax = '';
                $instance['post_type'] = $this->post_type = $p->post_type;
                $taxonomies = get_object_taxonomies($this->post_type, 'objects');
                switch (true) {
                    case $this->taxonomy && array_key_exists($this->taxonomy, $taxonomies) && is_object($taxonomies[$this->taxonomy]):

                        // get related terms and taxonomy_name
                        $wp_terms = get_the_terms($p->ID, $this->taxonomy);
                        $tax = $this->taxonomy;

                        if (!empty($wp_terms)) break;


                    case array_key_exists('post_tag', $taxonomies)
                        && is_object($taxonomies['post_tag'])
                        && get_the_terms($p->ID, 'post_tag') != false:

                        // get tag-related terms and taxonomy_name
                        $wp_terms = get_the_terms($p->ID, 'post_tag');
                        $tax = 'post_tag';
                        if (!empty($wp_terms)) break;


                    case array_key_exists('category', $taxonomies)
                        && is_object($taxonomies['category'])
                        && get_the_terms($p->ID, 'category') != false:

                        // get category-related terms and taxonomy_name
                        $wp_terms = get_the_terms($p->ID, 'category');
                        $tax = 'category';
                        if (!empty($wp_terms)) break;


                    default:
                        // get something-related terms and taxonomy_name
                        foreach ($taxonomies as $taxonomy_slug => $taxonomy) {
                            $wp_terms = get_the_terms($p->ID, $taxonomy_slug);
                            if (!empty($terms)) {
                                $tax = $taxonomy_slug;
                                break;
                            }
                        }
                }
                // set them to query instance
                if (!empty($wp_terms)) {
                    foreach ($wp_terms as &$wp_term) {
                        $terms[] = $wp_term->term_id;
                    }
                    $instance['term_ids_or_slugs'] = $this->term_ids_or_slugs = $terms;
                    $instance['taxonomy'] = $this->taxonomy = $tax;
                    $this->exclude_post_ids[] = $p->ID;
                }
            }
        }
    }
}
