<?php
/**
 * Util functions
 *
 * @author LeapIn Corp.
 * @since 1.0.0
 */
function leapin_get_css_file($file_name)
{
    if (LEAPIN_DEV) {
        $path = get_template_directory_uri() . '/library/css/src/';
    } else {
        $path = get_template_directory_uri() . '/library/css/dist/';
    }
    return $path . $file_name;
}

function leapin_get_js_file($file_name)
{
    if (LEAPIN_DEV) {
        $path = get_template_directory_uri() . '/library/js/src/';
    } else {
        $path = get_template_directory_uri() . '/library/js/dist/';
    }
    return $path . $file_name;
}


function leapin_get_yoast_primary_category_id($id = false)
{
    $cat = leapin_get_yoast_primary_category($id);
    return is_object($cat) ? $cat->term_id : false;
}

function leapin_get_yoast_primary_category($id = false)
{
    // SHOW YOAST PRIMARY CATEGORY, OR FIRST CATEGORY
    $category = $id ? get_the_category($id) : get_the_category();
    // If post has a category assigned.
    if ($category) {
        if (class_exists('WPSEO_Primary_Term')) {
            // Show the post's 'Primary' category, if this Yoast feature is available, & one is set
            $wpseo_primary_term = new WPSEO_Primary_Term('category', get_the_id());
            $wpseo_primary_term = $wpseo_primary_term->get_primary_term();
            $term = get_term($wpseo_primary_term);
            if (is_wp_error($term)) {
                // Default to first category id (not Yoast) if an error is returned
                $cat = $category[0];
            } else {
                // Yoast Primary category id
                $cat = $term;
            }
        } else {
            // Default, get the first category id in WP's list of assigned categories
            $cat = $category[0];
        }
        return $cat;
    }
}


/**
 * Converts Hex to RGBA
 *
 * @param $color
 * @param bool $opacity (0 ~ 1)
 * @return string
 */
function leapin_hex2rgba($color, $opacity = false)
{

    $default = 'rgb(0,0,0)';

    //Return default if no color provided
    if (empty($color))
        return $default;

    //Sanitize $color if "#" is provided
    if ($color[0] == '#') {
        $color = substr($color, 1);
    }

    //Check if color has 6 or 3 characters and get values
    if (strlen($color) == 6) {
        $hex = array($color[0] . $color[1], $color[2] . $color[3], $color[4] . $color[5]);
    } elseif (strlen($color) == 3) {
        $hex = array($color[0] . $color[0], $color[1] . $color[1], $color[2] . $color[2]);
    } else {
        return $default;
    }

    //Convert hexadec to rgb
    $rgb = array_map('hexdec', $hex);

    //Check if opacity is set(rgba or rgb)
    if ($opacity) {
        if (abs($opacity) > 1)
            $opacity = 1.0;
        $output = 'rgba(' . implode(",", $rgb) . ',' . $opacity . ')';
    } else {
        $output = 'rgb(' . implode(",", $rgb) . ')';
    }

    //Return rgb(a) color string
    return $output;
}

function leapin_get_eyecatch_image($size = 'leapin-medium', $post_id = false)
{
    $thumbnail_id = $post_id ? get_post_thumbnail_id($post_id) : get_post_thumbnail_id(get_the_ID());
    $img = wp_get_attachment_image_src($thumbnail_id, $size);
    if (empty($img)) {
        $default_img = LEAPIN_Theme_Mod::get_instance()
            ->get(LEAPIN_IDs::$leapin_single_default_image, LEAPIN_Theme_Mod::$default_leapin_single_default_image);
        if (isset($default_img)) {
            $img_size = leapin_get_image_width_and_height($default_img);
            $img[0] = $default_img;
            $img[1] = $img_size['width'];
            $img[2] = $img_size['height'];
        }
    }
    return $img;
}

function leapin_sanity_check($mixed_data)
{
    if (!empty ($mixed_data)) {
        return $mixed_data;
    }
    return "";
}

function leapin_adjust_color_brightness($hex, $steps)
{
    // Steps should be between -255 and 255. Negative = darker, positive = lighter
    $steps = max(-255, min(255, $steps));

    // Normalize into a six character long hex string
    $hex = str_replace('#', '', $hex);
    if (strlen($hex) == 3) {
        $hex = str_repeat(substr($hex, 0, 1), 2) . str_repeat(substr($hex, 1, 1), 2) . str_repeat(substr($hex, 2, 1), 2);
    }

    // Split into three parts: R, G and B
    $color_parts = str_split($hex, 2);
    $return = '#';

    foreach ($color_parts as $color) {
        $color = hexdec($color); // Convert to decimal
        $color = max(0, min(255, $color + $steps)); // Adjust color
        $return .= str_pad(dechex($color), 2, '0', STR_PAD_LEFT); // Make two char hex code
    }

    return $return;
}

function leapin_includes_site_url($url)
{
    // if url doesn't include site url
    if (strpos($url, site_url()) === false) {
        return false;
    } else {
        return true;
    }
}

function leapin_sanitize_text($str)
{
    return sanitize_text_field($str);
}

function leapin_sanitize_checkbox($input)
{
    return (bool)$input;
}

function leapin_sanitize_id_comma_text($comma_text)
{
    $removed_comma_text = trim(leapin_sanitize_text($comma_text));
    $removed_comma_text = preg_replace('/[^\d,]/i', '', $removed_comma_text);
    return $removed_comma_text;
}

function leapin_sanitize_number($int)
{
    return absint($int);
}

function leapin_sanitize_mock($value)
{
    return $value;
}

function leapin_sanitize_i_tag($value)
{
    return wp_kses($value,
        array(
            'i' => array(
                'class' => array()
            )
        ));
}

function leapin_is_barba_call()
{
    if (isset($_SERVER['HTLEAPIN_X_BARBA']) && $_SERVER['HTLEAPIN_X_BARBA'] == 'yes') {
        return true;
    }
    return false;
}


function leapin_sanitize_rgba_color($color)
{
    if ('' === $color)
        return '';

    if (false === strpos($color, 'rgba')) {
        $color = maybe_hash_hex_color($color);
        return $color;
    } else {
        $color = str_replace(' ', '', $color);
        $red = $green = $blue = $alpha = '';
        sscanf($color, 'rgba(%d,%d,%d,%f)', $red, $green, $blue, $alpha);
        return 'rgba(' . $red . ',' . $green . ',' . $blue . ',' . $alpha . ')';
    }
}

function leapin_mod_sr_only($key = false)
{
    $inst = LEAPIN_Theme_Mod::get_instance();
    return $inst->get($key) ? ' sr-only' : '';
}

/**
 * @param bool $key - theme mods hide key which should be hiding content if its value is true.
 * @return bool - true if it's visible
 */
function leapin_is_mod_visible($key = false)
{
    $inst = LEAPIN_Theme_Mod::get_instance();
    return $inst->get($key) ? false : true;
}

function leapin_get_old_browser_msg($SCRIPT_MODs)
{
    $SCRIPT_MODs['old_browser_notification'] =
        __('Oops, Your browser is too old. Please use newer one or alternative browser such as Chrome or Firefox!', 'pjax-blog');
    $SCRIPT_MODs['message_open_with_chrome'] =
        __('Open with Chrome', 'pjax-blog');
    $SCRIPT_MODs['message_open_with_firefox'] =
        __('Open with Firefox', 'pjax-blog');
    return $SCRIPT_MODs;
}

add_filter('leapin_add_script_properties', 'leapin_get_old_browser_msg');

function leapin_amp_head()
{
    do_action('leapin_amp_head');
}

function leapin_the_non_supported_ie_browser()
{
    $old_browser_notification = __('Oops, Your browser is too old. Please use newer one or alternative browser such as Chrome or Firefox!', 'pjax-blog');
    ?>
    <!--[if IE]>
    <p style="display: block!important;font-size: 20px;color: #000;">
        <?php echo $old_browser_notification;?>
    </p>
    <style>#container {
        display: none !important;
        background-color: #ffffff !important;
    }</style>
    <![endif]-->
    <?php
}

function leapin_set_custom_wp_head()
{
    $inst = LEAPIN_Theme_Mod::get_instance();

    // set author meta tag in head
    $author = LEAPIN_Single_Tags::get_user_name();
    if (!empty($author)):
        ?>
        <meta name="author" content="<?php echo $author; ?>">
    <?php
    endif;

    // set publisher meta tag in head tag
    $publisher = $inst->get(LEAPIN_IDs::$leapin_publisher_name, false);
    if (!empty($publisher)):?>
        <meta name="publisher" content="<?php echo $publisher; ?>"/>
    <?php
    endif;

    // set theme color
    $address_bar_color = $inst->get(LEAPIN_IDs::$leapin_other_address_bar_color, false);
    if (!empty($address_bar_color)): ?>
        <meta name="msapplication-TileColor" content="<?php echo $address_bar_color; ?>">
        <meta name="theme-color" content="<?php echo $address_bar_color; ?>">
    <?php
    endif;
}

add_action('wp_head', 'leapin_set_custom_wp_head');
add_action('leapin_amp_head', 'leapin_set_custom_wp_head');

function leapin_get_customizer_sec_url($section)
{
    $query['autofocus[section]'] = $section;
    $section_link = add_query_arg($query, admin_url('customize.php'));
    return esc_url($section_link);
}

function leapin_bg_image()
{
    $bool = !empty(get_background_image()) ? ' custom-background' : '';
    return $bool;
}

/**
 * get widgets in specific widget area
 *
 * @param bool $sidebar_id
 * @return array - Widget Arrays
 * @since 1.1.0
 * @see https://gist.github.com/kingkool68/3418186
 */
function leapin_get_widget_data_for($sidebar_id = false)
{
    global $wp_registered_sidebars, $wp_registered_widgets;
    // Holds the final data to return
    $output = array();
    $exists = false;
    // Loop over all of the registered sidebars looking for the one with the same name as $sidebar_name
    foreach ($wp_registered_sidebars as $sidebar) {
        if ($sidebar['id'] == $sidebar_id) {
            // We now have the Sidebar ID, we can stop our loop and continue.
            $exists = true;
            break;
        }
    }
    if (!$exists) {
        // There is no sidebar registered with the name provided.
        return $output;
    }
    // A nested array in the format $sidebar_id => array( 'widget_id-1', 'widget_id-2' ... );
    $sidebars_widgets = wp_get_sidebars_widgets();
    $widget_ids = $sidebars_widgets[$sidebar_id];
    if (!$widget_ids) {
        // Without proper widget_ids we can't continue.
        return array();
    }
    // Loop over each widget_id so we can fetch the data out of the wp_options table.
    foreach ($widget_ids as $id) {
        // The name of the option in the database is the name of the widget class.
        $option_name = $wp_registered_widgets[$id]['callback'][0]->option_name;
        // Widget data is stored as an associative array. To get the right data we need to get the right key which is stored in $wp_registered_widgets
        $key = $wp_registered_widgets[$id]['params'][0]['number'];
        $widget_data = get_option($option_name);
        // Add the widget data on to the end of the output array.
        $output[] = (object)$widget_data[$key];
    }
    return $output;
}

function leapin_get_neighbor_posts_tag()
{
    if (false) {
        /**
         * Do not delete this condition. Theme Check (http://wordpress.org/plugins/theme-check/)
         * is constantly telling me that it needs this code in the theme
         * even though we use get_adjacent_post instead. The REAL Code is from the end of this "if".
         */
        the_posts_pagination();
    }

    ob_start();
    ?>
    <div class="neighbor-posts" class="clearfix">
        <?php
        $prevpost = get_adjacent_post(false, '', true); // prev post
        $nextpost = get_adjacent_post(false, '', false); // next post
        if ($prevpost || $nextpost) { // if next or prev post exists
            leapin_get_neighbor_post($prevpost, true);
            leapin_get_neighbor_post($nextpost, false);
        } ?>
    </div>
    <?php
    return ob_get_clean();
}

function leapin_get_neighbor_post($neighborpost = false, $prev = true)
{
    $title = $prev ? __('PREV', 'pjax-blog') : __('NEXT', 'pjax-blog');
    $class = $prev ? 'prev' : 'next';
    if ($neighborpost) { // neighbor exists
        $img = leapin_get_eyecatch_image(array(100, 100), $neighborpost->ID);
        $img_tag = sprintf('<img src="%s" width="100" height="100"/>', $img[0]);
        echo '<a href="' . leapin_get_permalink($neighborpost->ID) . '" title="' . get_the_title($neighborpost->ID) . '" class="clearfix neighbor-posts__' . $class . '"><div class="neighbor-posts__' . $class . '-title">' . $title . '</div>'
            . $img_tag .
            '<p>' . get_the_title($neighborpost->ID) . '</p></a>';
    } else { // neighbor not exists
        echo '<a href="' . leapin_get_home_url() . '" title="home" class="clearfix neighbor-posts__' . $class . '"><div class="neighbor-posts__' . $class . '-title">HOME</div>'
            . '<i class="fas fa-home"></i>' .
            '<p>' . __('Go Back To Home', 'pjax-blog') . '</p></a>';
    }
}

function leapin_tag_cloud_custom_ex($output)
{
    // remove style attr in tag cloud widget
    $output = preg_replace('/\s*?style="[^"]+?"/i', '', $output);
    return $output;
}

add_filter('wp_tag_cloud', 'leapin_tag_cloud_custom_ex');

function leapin_is_comment_open()
{
    global $post;
    if (isset($post->comment_status)) {
        return $post->comment_status == 'open';
    }
    return false;
}

/**
 * Get image width and height
 *
 * @param $image_url
 * @return array|bool
 */
function leapin_get_image_width_and_height($image_url)
{
    if (!leapin_includes_site_url($image_url)) {
        return false;
    }
    $wp_upload_dir = wp_upload_dir();
    $uploads_dir = $wp_upload_dir['basedir'];
    $uploads_url = $wp_upload_dir['baseurl'];
    $image_file = str_replace($uploads_url, $uploads_dir, $image_url);
    $res = array();
    if (file_exists($image_file) &&
        ($imagesize = getimagesize($image_file))
    ) {
        $res['width'] = $imagesize[0];
        $res['height'] = $imagesize[1];
        return $res;
    } else {
        return false;
    }
}

function leapin_load_css_on_each_pjax()
{
    $css = '';
    echo '<style id="leapin_load_css_on_each_pjax">' . apply_filters('leapin_load_css_on_each_pjax', $css) . '</style>';
}

add_action('leapin_footer', 'leapin_load_css_on_each_pjax');

function leapin_footer()
{
    do_action('leapin_footer');
}

/*********************
 * SET CUSTOM FIELD TO TAX
 *********************/
function leapin_add_custom_field_to_taxonomy()
{
    // A callback function to add a custom field to our "colors" taxonomy
    function leapin_taxonomy_custom_fields($tag)
    {
        // Check for existing taxonomy meta for the term you're editing
        if (is_object($tag)) :
            $t_id = $tag->term_id; // Get the ID of the term you're editing
            $term_meta = get_option("taxonomy_term_$t_id"); // Do the check
            ?>
            <tr class="form-field">
                <th scope="row" valign="top">
                    <label for="tax_color"><?php _e('Theme color', 'pjax-blog'); ?></label>
                </th>
                <td>
                    <?php
                    leapin_admin_output_alpha_color_picker_field(
                        'alpha-color-picker',
                        'term_meta[tax_color]',
                        $term_meta['tax_color'] ? $term_meta['tax_color'] : '',
                        'true',
                        '',
                        'true'
                    );
                    ?>
                    <p class="description"><?php _e('Picked color will be used in this theme.', 'pjax-blog'); ?></p>
                </td>
            </tr>
        <?php else: ?>
            <div class="form-field terms-tax_color-wrap">
                <label for="tax_color"><?php _e('Theme color', 'pjax-blog'); ?></label>
                <?php
                leapin_admin_output_alpha_color_picker_field(
                    'alpha-color-picker',
                    'term_meta[tax_color]',
                    '',
                    'true',
                    '',
                    'true'
                );
                ?>
                <p class="description"><?php _e('Picked color will be used in this theme.', 'pjax-blog'); ?></p>
            </div>
        <?php endif;
    }

    // A callback function to save our extra taxonomy field(s)
    function leapin_save_taxonomy_custom_fields($term_id)
    {
        if (isset($_POST['term_meta'])) {
            $t_id = $term_id;
            $term_meta = get_option("taxonomy_term_$t_id");
            $cat_keys = array_keys($_POST['term_meta']);
            foreach ($cat_keys as $key) {
                if (isset($_POST['term_meta'][$key])) {
                    $term_meta[$key] = $_POST['term_meta'][$key];
                }
            }
            //save the option array
            update_option("taxonomy_term_$t_id", $term_meta);
        }
    }

    global $leapin_vars;
    $taxonomies = $leapin_vars['taxonomies'];

    foreach ($taxonomies as $tax) {
        $tax_name = $tax->name;
        // Add the fields to taxonomies, using our callback function
        add_action("${tax_name}_edit_form_fields", 'leapin_taxonomy_custom_fields', 10, 2);

        // Save the changes made on taxonomies, using our callback function
        add_action("edited_${tax_name}", 'leapin_save_taxonomy_custom_fields', 10, 2);
    }

}

// @todo implement in latter version
//add_action('wp_loaded', 'add_leapin_custom_field_to_taxonomy', 11);


/**
 * Check if the current theme is pro version
 *
 * @return bool true in pro edition
 */
function leapin_is_pro()
{
    return class_exists('LEAPIN_Theme_Grade') && !LEAPIN_Theme_Grade::is_free_mode();
}

/**
 * Implode Array Values
 *
 * @param string $glue
 * @param array $arry
 * @return string glued array values
 * return "" if $arry is empty.
 * @since 1.6.6
 */
function leapin_implode($glue, $arry)
{
    $rtn = "";
    if (is_array($arry)) {
        if (count($arry) == 1) {
            $rtn = $arry[0];
        } else if (count($arry) > 1) {
            $rtn = implode($glue, $arry);
        }
    }
    return $rtn;
}

if (is_plugin_active('siteorigin-panels/siteorigin-panels.php')) {
    add_filter('siteorigin_panels_row_style_css', 'leapin_change_background_size', 10, 2);
    add_filter('siteorigin_panels_cell_style_css', 'leapin_change_background_size', 10, 2);
    add_filter('siteorigin_panels_widget_style_css', 'leapin_change_background_size', 10, 2);
    function leapin_change_background_size($css, $style)
    {
        if (!empty($css['background-image'])) {
            $url = wp_get_attachment_image_src($style['background_image_attachment'], 'large');

            if (!empty($url)) {
                $css['background-image'] = 'url(' . (is_array($url) ? $url[0] : $url) . ')';
            }
        }
        return $css;
    }

    add_filter('siteorigin_panels_row_style_attributes', 'leapin_change_background_size_attr', 10, 2);
    add_filter('siteorigin_panels_cell_style_attributes', 'leapin_change_background_size_attr', 10, 2);
    add_filter('siteorigin_panels_widget_style_attributes', 'leapin_change_background_size_attr', 10, 2);
    function leapin_change_background_size_attr($attributes, $style)
    {
        if (!empty($style['background_image_attachment'])) {
            $url = wp_get_attachment_image_src($style['background_image_attachment'], 'large');

            if (
                !empty($url) &&
                ($style['background_display'] == 'parallax' || $style['background_display'] == 'parallax-original')
            ) {
                $parallax_args = array(
                    'backgroundUrl' => $url[0],
                    'backgroundSize' => array($url[1], $url[2]),
                    'backgroundSizing' => $style['background_display'] == 'parallax-original' ? 'original' : 'scaled',
                    'limitMotion' => siteorigin_panels_setting('parallax-motion') ? floatval(siteorigin_panels_setting('parallax-motion')) : 'auto',
                );
                $attributes['data-siteorigin-parallax'] = json_encode($parallax_args);
            }
        }
        return $attributes;
    }
}

function leapin_get_home_url()
{
    return esc_url(home_url());
}

function leapin_get_permalink($post = 0)
{
    return esc_url(get_permalink($post));
}

function leapin_get_author_credit()
{
    if (!leapin_is_pro()) {
        printf('<a href="' . esc_url('https://leap-in.com/') . '" rel="author" target="_blank">%s</a>.', __('Pjax Blog Theme Developed by LeapIn', 'pjax-blog'));
    }
}

add_action('leapin_loc_footer_author_credit', 'leapin_get_author_credit');

function leapin_str_replace_n($search, $replace, $str, $n)
{
    if (preg_match_all($search, $str, $m, PREG_OFFSET_CAPTURE) > $n) {
        $str = substr_replace($str, $replace, $m[0][$n][1], strlen($m[0][$n][0]));
    }
    return $str;
}

/**
 * Check if Pjax Blog Bundle is activated
 *
 * @return bool - true activated
 */
function leapin_is_bundle(){
    return is_plugin_active('leapin-bundle/leapin-bundle.php');
}